<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Controller;
use App\Models\User;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Mail;
use Illuminate\Support\Facades\Validator;
use Illuminate\Support\Str;
use Illuminate\Support\Facades\DB;
use App\Mail\UserInvitationMail;

class UserBulkUploadController extends Controller
{
    /**
     * Show the bulk upload form
     */
    public function showUploadForm()
    {
        return view('admin.users.bulk_upload');
    }

    /**
     * Process the CSV upload
     */
    public function processUpload(Request $request)
    {
        // Validate the request
        $request->validate([
            'csv_file' => 'required|file|mimes:csv,txt|max:10240',
        ]);

        // Process the CSV file
        $path = $request->file('csv_file')->getRealPath();
        $data = array_map('str_getcsv', file($path));
        
        // Remove header row if present
        $headers = array_shift($data);

        // Expected headers
        $expectedHeaders = [
            'name', 'email', 'department', 'employee_id', 'credit_limit', 
            'role', 'phone', 'position'
        ];

        // Validate headers
        $missingHeaders = array_diff($expectedHeaders, $headers);
        if (count($missingHeaders) > 0) {
            return redirect()->back()
                ->with('error', 'CSV file is missing required columns: ' . implode(', ', $missingHeaders));
        }

        // Start database transaction
        DB::beginTransaction();
        
        try {
            $successful = 0;
            $failed = 0;
            $errors = [];

            foreach ($data as $index => $row) {
                $rowData = array_combine($headers, $row);
                
                // Validate row data
                $validator = Validator::make($rowData, [
                    'name' => 'required|string|max:255',
                    'email' => 'required|email|unique:users,email',
                    'department' => 'required|string|max:100',
                    'employee_id' => 'required|string|max:50|unique:users,employee_id',
                    'credit_limit' => 'required|numeric|min:0',
                    'role' => 'nullable|in:admin,employee,manager',
                    'phone' => 'nullable|string|max:20',
                    'position' => 'nullable|string|max:100',
                ]);

                if ($validator->fails()) {
                    $rowNumber = $index + 2; // +1 for 0-based array, +1 for header row
                    $errors[] = "Row {$rowNumber}: " . implode(', ', $validator->errors()->all());
                    $failed++;
                    continue;
                }

                // Create temporary password
                $tempPassword = Str::random(10);
                
                // Create new user
                $user = User::create([
                    'name' => $rowData['name'],
                    'email' => $rowData['email'],
                    'password' => Hash::make($tempPassword),
                    'department' => $rowData['department'],
                    'employee_id' => $rowData['employee_id'],
                    'credit_limit' => $rowData['credit_limit'],
                    'credit_used' => 0,
                    'status' => 'active',
                    'is_first_login' => true,
                    'role' => $rowData['role'] ?? 'employee',
                    'phone' => $rowData['phone'] ?? null,
                    'position' => $rowData['position'] ?? null,
                ]);
                
                // Send invitation email
                try {
                    Mail::to($user->email)->send(new UserInvitationMail($user, $tempPassword));
                    $successful++;
                } catch (\Exception $e) {
                    $errors[] = "Email failed to send to {$user->email}: {$e->getMessage()}";
                }
            }
            
            // Commit transaction
            DB::commit();
            
            // Return success message
            return redirect()->back()
                ->with('success', "Bulk upload completed: {$successful} users added successfully, {$failed} failed.")
                ->with('errors', $errors);
                
        } catch (\Exception $e) {
            // Rollback transaction
            DB::rollBack();
            
            return redirect()->back()
                ->with('error', 'An error occurred during the upload: ' . $e->getMessage());
        }
    }

    /**
     * Download CSV template
     */
    public function downloadTemplate()
    {
        $headers = [
            'name', 'email', 'department', 'employee_id', 'credit_limit', 
            'role', 'phone', 'position'
        ];
        
        $sampleData = [
            [
                'John Doe',
                'john.doe@ampcontrol.com',
                'Engineering',
                'EMP001',
                '500.00',
                'employee',
                '0412345678',
                'Engineer'
            ],
            [
                'Jane Smith',
                'jane.smith@ampcontrol.com',
                'Finance',
                'EMP002',
                '750.00',
                'manager',
                '0487654321',
                'Finance Manager'
            ]
        ];
        
        $output = fopen('php://temp', 'w');
        fputcsv($output, $headers);
        
        foreach ($sampleData as $row) {
            fputcsv($output, $row);
        }
        
        rewind($output);
        $csvContent = stream_get_contents($output);
        fclose($output);
        
        return response($csvContent)
            ->header('Content-Type', 'text/csv')
            ->header('Content-Disposition', 'attachment; filename="user_upload_template.csv"');
    }
}
